@@ -3,6 +3,8 @@ use Mojo::Base 'Mojolicious::Controller';
 
 =pod
 
+=encoding utf8
+
 =head1 Manual
 
     $ perl -e "use Mojo::Base 'Mojolicious'; __PACKAGE__->new()->start(); sub startup {shift->routes->route('/')->to('install#manual', namespace=>'Mojolicious::Plugin::RoutesAuthDBI');}" get / 2>/dev/null
@@ -82,7 +84,7 @@ $ perl test-app.pl daemon
 5. Go to trust url for admin-user creation :
 ------------
 
-$ perl test-app.pl get /<pluginconf->{admin}{prefix}>/<pluginconf->{admin}{trust}>/user/new/<new admin login>/<admin pass>
+$ perl test-app.pl get /<pluginconf->{admin}{prefix}>/<pluginconf->{admin}{trust}>/admin/new/<new admin login>/<admin pass>
 
 User will be created and assigned to role 'Admin' . Role 'Admin' assigned to pseudo-route that has access to all routes of this controller!
 
@@ -135,17 +137,31 @@ CREATE SEQUENCE ID;-- one sequence for all tables id
 CREATE TABLE routes (
     id integer default nextval('ID'::regclass) not null primary key,
     ts timestamp without time zone default now() not null,
-    request character varying null,
-    namespace character varying null,
-    controller character varying not null,
-    action character varying null,
-    name character varying not null,
-    descr text,
-    auth bit(1),
-    disable bit(1),
-    order_by int
+    request character varying not null,
+    name character varying not null unique,
+    descr text null,
+    auth bit(1) null,
+    disable bit(1) null,
+    order_by int null
 );
 
+create table controllers (
+      id integer default nextval('ID'::regclass) not null primary key,
+      ts timestamp without time zone default now() not null,
+      namespace character varying null,
+      controller character varying not null,
+      descr text null,
+      unique(namespace, controller)
+    );
+
+create table actions (
+      id integer default nextval('ID'::regclass) not null primary key,
+      ts timestamp without time zone default now() not null,
+      action character varying not null,
+      callback text null,
+      descr text null
+    );
+
 create table users (
         id int default nextval('ID'::regclass) not null  primary key,
         ts timestamp without time zone default now() not null,
@@ -182,6 +198,8 @@ drop table refs;
 drop table users;
 drop table roles;
 drop table routes;
+drop table controllers;
+drop table actions;
 drop sequence ID;
 
 TXT
@@ -196,6 +214,8 @@ delete from refs;
 delete from users;
 delete from roles;
 delete from routes;
+delete from controllers;
+delete from actions;
 
 TXT
 }
